<?php
/**
 * Glop Admin Footer
 *
 * @author   Daniel Ruiz
 * @category Admin
 * @package GLOP/wooglop
 * @since 1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Glop_Admin
 */
class Wooglop_Admin {

	/**
	 * Glop_Admin constructor.
	 */
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'wooglop_register_custom_menu_page' ) );
		add_action( 'wooglop_admin_field_hidden', array( 'Wooglop_Admin_Settings', 'get_hidden_field' ) );
		add_action( 'wooglop_admin_field_glop_lista_usuarios', array( 'Wooglop_Admin_Settings', 'get_glop_lista_usuarios' ) );
		/**
		 * 18 Marzo 2021 #367
		 * Nuevo campo SKU glop.
		 */
		add_action( 'woocommerce_product_options_sku', array( $this, 'woocommerce_product_options_sku_glop' ) );
		add_action( 'save_post', array( $this, 'woocommerce_save_glop_product' ) );
		add_action( 'woocommerce_variation_options', array( $this, 'woocommerce_glop_variation_options' ), PHP_INT_MAX, 3 );
		add_action( 'woocommerce_ajax_save_product_variations', array( $this, 'woocommerce_save_glop_variation' ) );
	}

	/**
	 * Glop Register Custom Menu Page.
	 */
	public function wooglop_register_custom_menu_page() {
		add_menu_page(
			__( 'WooGlop', 'wooglop' ),
			__( 'WooGlop', 'wooglop' ),
			'manage_options',
			'wooglop',
			array( $this, 'wooglop_print_admin_page' ),
			plugins_url( 'wooglop/icon.png' ),
			10
		);
	}

	/**
	 * 18 Marzo 2021 #367
	 * Nuevo campo SKU glop.
	 */
	public function woocommerce_product_options_sku_glop() {
		// SI está activa la opción de glop entonces mostrar caja, si no, salir.
		if ( ! Wooglop_Admin::sku_glop_enabled() ) {
			return;
		}
		/** @var WC_Product $product_object */
		global $product_object;
		woocommerce_wp_text_input(
			array(
				'id'          => '_sku_glop',
				'value'       => get_post_meta( $product_object->get_id(), 'sku_glop', true ),
				'label'       => '<abbr title="' . esc_attr__( 'SKU Glop', 'woocommerce' ) . '">' . esc_html__( 'SKU GLOP', 'woocommerce' ) . '</abbr>',
				'desc_tip'    => true,
				'description' => __( 'SKU Glop', 'woocommerce' ),
			)
		);
	}

	/**
	 * @param int     $loop
	 * @param array   $variation_data
	 * @param WP_Post $variation
	 */
	public function woocommerce_glop_variation_options( $loop, $variation_data, $variation ) {

		// SI está activa la opción de glop entonces mostrar caja, si no, salir.
		if ( ! Wooglop_Admin::sku_glop_enabled() || ( 'product_variation' !== $variation->post_type ) ) {
			return;
		}
		woocommerce_wp_text_input(
			array(
				'id'            => "variable_sku_glop{$loop}",
				'name'          => "variable_sku_glop[{$loop}]",
				'value'         => get_post_meta( $variation->ID, 'sku_glop', true ),
				'placeholder'   => get_post_meta( $variation->ID, 'sku_glop', true ),
				'label'         => '<abbr title="' . esc_attr__( 'SKU Glop', 'woocommerce' ) . '">' . esc_html__( 'SKU Glop', 'woocommerce' ) . '</abbr>',
				'desc_tip'      => true,
				'description'   => __( 'SKU Glop', 'woocommerce' ),
				'wrapper_class' => 'form-row',
			)
		);
	}

	/**
	 * @param int $product_id
	 */
	public function woocommerce_save_glop_variation( $product_id ) {

		// SI está activa la opción de glop entonces mostrar caja, si no, salir.
		if ( ! Wooglop_Admin::sku_glop_enabled() ) {
			return;
		}
		if ( isset( $_POST['variable_post_id'] ) ) {
			$max_loop   = max( array_keys( wp_unslash( $_POST['variable_post_id'] ) ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

			for ( $i = 0; $i <= $max_loop; $i++ ) {
				if ( ! isset( $_POST['variable_post_id'][ $i ] ) ) {
					continue;
				}
				$variation_id = absint( $_POST['variable_post_id'][ $i ] );
				update_post_meta( $variation_id, 'sku_glop', $_POST['variable_sku_glop'][ $i ] );
			}
		}
	}

	/**
	 * @param $post_id
	 * @param false $post
	 */
	public function woocommerce_save_glop_product( $post_id, $post = false ) {
		if ( ! Wooglop_Admin::sku_glop_enabled() ) {
			return;
		}
		global $post;
		$post_type = $post ? $post->post_type : get_post_type( $post_id );
		if ( 'product' === $post_type ) {
			$sku_glop = $_POST['_sku_glop'];
			update_post_meta( $post_id, 'sku_glop', $sku_glop );
		}
	}

	/**
	 * Glop Print Admin Page.
	 */
	public function wooglop_print_admin_page() {
		$this->glop_save_glop_ajustes();
		include WOOGLOP_PLUGIN_ADMIN_TPL_PATH . 'glop-admin-page.php';
	}

	/**
	 * Glop Save Glop Ajustes.
	 */
	public function glop_save_glop_ajustes() {
		if (
			isset( $_POST['_wpnonce'] ) &&
			wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ), sanitize_key( 'glop-settings' ) ) &&
			array_key_exists( 'save', $_POST )
		) {
			$post = $_POST;
			if ( array_key_exists( 'glop_activar_servicio', $post ) ) {
				$post['glop_activar_servicio'] = ( '1' === $post['glop_activar_servicio'] ) ? 'yes' : 'no';
			}
			if ( array_key_exists( 'glop_activar_debug', $post ) ) {
				$post['glop_activar_debug'] = ( '1' === $post['glop_activar_debug'] ) ? 'yes' : 'no';
			}
			if ( array_key_exists( 'glop_gestionar_inventario', $post ) ) {
				$post['glop_gestionar_inventario'] = ( '1' === $post['glop_gestionar_inventario'] ) ? 'yes' : 'no';
			}
			/**
			 * 18 Marzo 2021 #366
			 * Nuevo campo SKU glop.
			 */
			if ( array_key_exists( 'glop_activar_sku', $post ) ) {
				$post['glop_activar_sku'] = ( '1' === $post['glop_activar_sku'] ) ? 'yes' : 'no';
			}
			$this->glop_save_recursive_save_ajustes( $post );
		}
	}

	/**
	 * Glop Save Recursive Save Ajustes.
	 *
	 * @param null $params Params.
	 */
	public function glop_save_recursive_save_ajustes( $params = null ) {
		foreach ( $params as $key => $item ) {
			if ( substr( $key, 0, 4 ) === 'glop' ) {
				update_option( $key, $item );
			}
		}
	}

	/**
	 * Is Glop Actived.
	 *
	 * @return bool
	 */
	public static function glop_actived() {
		return get_option( 'glop_activar_servicio', 'yes' ) === 'yes';
	}

	/**
	 * Is Glop in debug mode.
	 *
	 * @return bool
	 */
	public static function glop_in_debug() {
		return get_option( 'glop_activar_debug', 'no' ) === 'yes';
	}
	/**
	 * Is Glop in debug mode.
	 *
	 * @return bool
	 */
	public static function glop_gestionar_inventario() {
		$key = 'glop_gestionar_inventario' ;
		if ( ! get_option( $key ) ) {
			update_option( $key, 'yes' );
		}
		return get_option( $key, 'yes' ) === 'yes';
	}

	/**
	 * Is Glop SKU enabled. #368 18 marzo 2021
	 *
	 * @return bool
	 */
	public static function sku_glop_enabled() {
		$key = 'glop_activar_sku' ;
		if ( ! get_option( $key ) ) {
			update_option( $key, 'no' );
		}
		return get_option( $key, 'no' ) === 'yes';
	}

	/**
	 * Product by Glop SKU. #368 18 marzo 2021
	 *
	 * @param int $sku_glop
	 *
	 * @return WC_Product|null
	 */
	public static function glop_get_product_by_sku_glop( $sku_glop, $get_the_id = false ) {
		$args = array(
			'post_type'  => array( 'product', 'product_variation' ),
			'meta_query' => array(
				array(
					'key'   => 'sku_glop',
					'value'	=>	$sku_glop
				)
			)
		);
		$my_query = new WP_Query( $args );
		if( $my_query->have_posts() ) {
			$my_query->the_post();
			global $post;

			if ( get_the_ID() ) {
				if ( $get_the_id ) {
					return get_the_ID();
				}
				if( 'product' === $post->post_type ) {
					return new WC_Product( get_the_ID() );
				} else {
					return new WC_Product_Variation( get_the_ID() );
				}
			}
		}

		return null;
	}

	/**
	 * Guardar el sku_glop
	 *
	 * @param int $post_id
	 * @param int $sku_glop
	 */
	public static function glop_save_sku_glop( $post_id, $sku_glop ) {
		update_post_meta( $post_id, 'sku_glop', $sku_glop );
	}

	/**
	 * Get SKU GLOP
	 * @param $post_id
	 *
	 * @return mixed
	 */
	public static function glop_sku_glop( $post_id ) {
		return get_post_meta( $post_id, 'sku_glop', true );
	}

	/**
	 * Glop usuario.
	 *
	 * @return mixed|null
	 */
	public static function glop_usuario() {
		return get_option( 'glop_usuario', null );
	}

	/**
	 * Glop usuario.
	 *
	 * @return mixed|null
	 */
	public static function glop_usuarios() {
		return get_option( 'wooglop_usuarios', null );
	}

	/**
	 * Glop Password.
	 *
	 * @return mixed|null
	 */
	public static function glop_password() {
		return get_option( 'glop_pass', '' );
	}

	/**
	 * Glop Key.
	 *
	 * @return mixed|null
	 */
	public static function glop_key() {
		return get_option( 'glop_key', '' );
	}

	/**
	 * Get payments methods.
	 *
	 * @return WC_Payment_Gateway[]
	 */
	public static function get_payments_methods() {
		if ( ! class_exists( 'WooCommerce' ) ) {
			return array();
		}

		// Todos los métodos instalados aunque no estén activos
		// return WC()->payment_gateways->get_payment_gateway_ids();
		// Solo los métodos activos.
		return WC()->payment_gateways->get_available_payment_gateways();
	}

	/**
	 * Get shipment methods.
	 *
	 * @return array
	 */
	public static function get_shipment_methods() {
		if ( ! class_exists( 'WooCommerce' ) ) {
			return array();
		}

		return WC()->shipping()->get_shipping_methods();
	}

	/**
	 * Get tax methods
	 *
	 * @return array
	 */
	public static function get_tax_methods() {
		if ( ! class_exists( 'WooCommerce' ) ) {
			return array();
		}

		$tax_classes             = array();
		$tax_classes['standard'] = __( 'Standard rate', 'wooglop' );
		$classes                 = WC_Tax::get_tax_classes();

		foreach ( $classes as $class ) {
			$tax_classes[ sanitize_title( $class ) ] = $class;
		}

		return $tax_classes;
	}
}
