<?php
/**
 * REST API: Wooglop_REST_Glop_Order_Controller class
 *
 * @package Glop
 * @subpackage wooglop
 * @since 1.0.0
 */

/**
 * Core class to GLOP via the REST API.
 *
 * @since 1.0.0
 *
 * @see Wooglop_WP_REST_Controller
 */
class Wooglop_REST_Glop_Order_Controller extends Wooglop_WP_REST_Controller {

	/**
	 * Registers the routes for the objects of the controller.
	 *
	 * @since 1.0.0
	 *
	 * @see register_rest_route()
	 */
	public function register_routes() {
		$this->rest_base = 'pedidos';
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				'args'   => array(),
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'get_orders' ),
					'permission_callback' => array( $this, 'get_items_permissions_check' ),
					'args'                => $this->get_collection_params(),
				),
				'schema' => array( $this, 'get_public_item_schema' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base . '/(?P<id>[a-zA-Z0-9-]+)',
			array(
				'args'   => array(
					'id' => array(
						'description' => __( 'Unique identifier for the resource.', 'wooglop' ),
						'type'        => 'string',
					),
				),
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'create_item' ),
					'permission_callback' => array( $this, 'get_items_permissions_check_2' ),
					'args'                => array(
						'context' => $this->get_context_param( array( 'default' => 'view' ) ),
					),
				),
				'schema' => array( $this, 'get_public_item_schema' ),
			)
		);
	}

	/**
	 * Crear JSON object
	 *
	 * @since 1.0.0
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function create_item( $request ) {
		$this->log_request( $request );
		try {
			$order_id = $request['id'];
			/** @var WC_Order $order */
			$order    = wc_get_order( $order_id );
			$order->update_status( 'completed' );
			$response = rest_ensure_response( $order );
			$response->set_status( 200 );

		} catch ( Wooglop_Exception $e ) {
			return new WP_Error( $e->getErrorCode(), $e->getMessage(), array( 'status' => $e->getCode() ) );
		} catch ( WC_REST_Exception $e ) {
			return new WP_Error( $e->getErrorCode(), $e->getMessage(), array( 'status' => $e->getCode() ) );
		}
	}

	/**
	 * Fecha1 y Fecha2 deben estar en timestamp
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 *
	 * @throws Wooglop_Exception Excepcion.
	 */
	public function get_orders( $request ) {
		$fecha1 = $request->get_param( 'fecha1' );
		$fecha2 = $request->get_param( 'fecha2' );

		try {
			$fecha1 = new DateTime( "@$fecha1" );
			$fecha2 = new DateTime( "@$fecha2" );

			$filter = array();
			$query  = $this->query_orders(
				array(
					'created_at_min' => $fecha1->format( 'Y-m-d H:i:s' ),
					'created_at_max' => $fecha2->format( 'Y-m-d H:i:s' ),
				)
			);

			$orders = array();


			foreach ( $query->posts as $order_id ) {

				if ( ! $this->is_readable( $order_id ) ) {
					continue;
				}
				if ( '62.57.20.247' === getenv( 'REMOTE_ADDR' ) ) {
					if ( 4720 !== $order_id ) {
//						continue;
					}
				}
				$orders[] = current( $this->get_order( $order_id, null, $filter ) );
			}

			foreach ( $orders as &$order ) {
				$d                     = new DateTime( $order['created_at'] );
				$order['created_at']   = $d->format( 'Y' ) !== '1970' ? $d->getTimestamp() : '';
				$d                     = new DateTime( $order['updated_at'] );
				$order['updated_at']   = $d->format( 'Y' ) !== '1970' ? $d->getTimestamp() : '';
				$d                     = new DateTime( $order['completed_at'] );
				$order['completed_at'] = $d->format( 'Y' ) !== '1970' ? $d->getTimestamp() : '';

				$order['total']          = floatval( $order['total'] ) * 100;
				$order['subtotal']       = floatval( $order['subtotal'] ) * 100;
				$order['total_tax']      = floatval( $order['total_tax'] ) * 100;
				$order['total_shipping'] = floatval( $order['total_shipping'] ) * 100;
				$order['cart_tax']       = floatval( $order['cart_tax'] ) * 100;
				$order['shipping_tax']   = floatval( $order['shipping_tax'] ) * 100;
				$order['total_discount'] = floatval( $order['total_discount'] ) * 100;
				$order['shipping_lines']
					['total']            = isset( $order['shipping_lines']['total'] ) ? floatval( $order['shipping_lines']['total'] ) * 100 : 0;
				$order['tax_lines']
					['total']            = isset( $order['tax_lines']['total'] ) ? floatval( $order['tax_lines']['total'] ) * 100 : 0;

				// Por producto, añadir el rate aplicado.
				if ( array_key_exists( 'line_items', $order ) ) {
					foreach ( $order['line_items'] as &$line_item ) {
						$t = WC_Tax::find_rates(
							array(
								'city'      => $order['shipping_address']['city'],
								'state'     => $order['shipping_address']['state'],
								'postcode'  => $order['shipping_address']['postcode'],
								'country'   => $order['shipping_address']['country'],
								'tax_class' => $line_item['tax_class'],
							)
						);

						if ( $t ) {
							$t                = array_values( $t );
							$t                = $t[0];
							$line_item['tax'] = number_format( $t['rate'], 2 );
						}
						$s = null;

						$es_variacion = wc_get_order_item_meta( absint( $line_item['id'] ), '_variation_id', true );
						if ( ! empty( $es_variacion ) ) {
							$variacion               = new WC_Product_Variation( $es_variacion );
							$line_item['product_id'] = $variacion->get_parent_id();
						}

						try {
							$s = new WC_Product( $line_item['product_id'] );

						} catch ( Exception $f ) {
							// Queremos seguir adelante.
						}

						$p = get_post_meta( absint( $line_item['product_id'] ), '_product_addons', true );

						if ( ! empty ( $p ) || ! empty( $es_variacion ) ) {

							$parametros_extras = $this->extract_articulos_extras( $line_item, $p );
							$line_item = array_merge( $line_item, $parametros_extras );
							unset( $line_item['meta'] );

						}

						// Multiplicación por 100 de todos los precios.
						$line_item['subtotal']     = floatval( $line_item['subtotal'] ) * 100;
						$line_item['subtotal_tax'] = floatval( $line_item['subtotal_tax'] ) * 100;
						$line_item['total']        = floatval( $line_item['total'] ) * 100;
						$line_item['total_tax']    = floatval( $line_item['total_tax'] ) * 100;
						$line_item['price']        = floatval( $line_item['price'] ) * 100;
						$line_item['tax']          = floatval( $line_item['tax'] ) * 100;


						try {
							if ( empty( $s ) ) {
								$v              = new WC_Product_Variation( $line_item['product_id'] );
								$product_parent = $v->get_parent_data();
								if ( ! empty( $product_parent['sku'] ) ) {
									$line_item['product_sku'] = $product_parent['sku'];
								} else {
									$line_item['product_sku'] = $line_item['sku'];
									if( Wooglop_Admin::sku_glop_enabled() ) {
										$line_item['product_sku'] = Wooglop_Admin::glop_sku_glop( $es_variacion );
									}
								}
							} else {
								$line_item['product_sku'] = $line_item['sku'];
							}
						} catch ( Exception $f ) {
							// Queremos seguir adelante.
						}
						if ( $s && $s->is_type( 'simple' ) && empty( $es_variacion ) ) {
							$line_item['sku'] = '0';
						}

						if ( ! empty( $es_variacion ) ) {
							$line_item['product_sku'] = $s->get_sku();
						}

						if( Wooglop_Admin::sku_glop_enabled() ) {
							$line_item['product_sku'] = Wooglop_Admin::glop_sku_glop( $line_item['product_id'] );
							// DANIEL RUIZ 19 Abril 2021. No se envía el sku.
							if ( ! empty( $es_variacion ) ) {
								$line_item['sku'] = Wooglop_Admin::glop_sku_glop( $es_variacion );
							}
						}
					}
				}

				$byconsolewooodt_delivery_date = get_post_meta( $order['id'], 'byconsolewooodt_delivery_date', true );
				$byconsolewooodt_delivery_time = get_post_meta( $order['id'], 'byconsolewooodt_delivery_time', true );
				$byconsolewooodt_delivery_type = get_post_meta( $order['id'], 'byconsolewooodt_delivery_type', true );

				if ( 'solo_establecimiento' === $byconsolewooodt_delivery_type ) {
					$byconsolewooodt_delivery_type = 'local';
				}

				// type: take_away o levering.
				if ( ! empty( $byconsolewooodt_delivery_date ) ) {
					$order['entrega']['fecha_entrega'] = $byconsolewooodt_delivery_date;
				}
				if ( ! empty( $byconsolewooodt_delivery_time ) ) {
					$order['entrega']['hora_entrega'] = $byconsolewooodt_delivery_time;
				}
				if ( ! empty( $byconsolewooodt_delivery_type ) ) {
					$order['entrega']['tipo_entrega'] = $byconsolewooodt_delivery_type;
				}

				$alias = get_post_meta( $order['id'], 'byconsolewooodt_chekout_page_alias_texto', true );
				if ( ! empty( $alias ) ) {
					$order['alias'] = $alias;
				}

				$id_mesa_selected = get_post_meta( $order['id'], 'id_mesa_selected', true );
				if ( ! empty( $id_mesa_selected ) ) {
					$order['id_mesa_selected'] = $id_mesa_selected;
				}
			}

			return array(
				'orders' => $orders
			);

		} catch ( Exception $e ) {
			return new WP_Error(
				'glop_api_pedidos_error_fechas',
				/* translators: %1$s: accion */
				__( 'Error en las fechas', 'wooglop' ),
				array( 'status' => 400 )
			);
		}
	}


	/**
	 * Get the order for the given ID
	 *
	 * @param int   $id the order ID.
	 * @param null  $fields fields.
	 * @param array $filter filters.
	 *
	 * @return array|WP_Error
	 * @since 1.0
	 */
	public function get_order( $id, $fields = null, $filter = array() ) {

		// Get the decimal precession.
		$dp         = ( isset( $filter['dp'] ) ? intval( $filter['dp'] ) : 2 );
		$order      = wc_get_order( $id );
		$order_data = array(
			'id'                        => $order->get_id(),
			'order_number'              => $order->get_order_number(),
			'created_at'                => $this->format_datetime( $order->get_date_created() ? $order->get_date_created()->getTimestamp() : 0, false, false ), // API gives UTC times.
			'updated_at'                => $this->format_datetime( $order->get_date_modified() ? $order->get_date_modified()->getTimestamp() : 0, false, false ), // API gives UTC times.
			'completed_at'              => $this->format_datetime( $order->get_date_completed() ? $order->get_date_completed()->getTimestamp() : 0, false, false ), // API gives UTC times.
			'status'                    => $order->get_status(),
			'currency'                  => $order->get_currency(),
			'total'                     => wc_format_decimal( $order->get_total(), $dp ),
			'subtotal'                  => wc_format_decimal( $order->get_subtotal(), $dp ),
			'total_line_items_quantity' => $order->get_item_count(),
			'total_tax'                 => wc_format_decimal( $order->get_total_tax(), $dp ),
			'total_shipping'            => wc_format_decimal( $order->get_shipping_total(), $dp ),
			'cart_tax'                  => wc_format_decimal( $order->get_cart_tax(), $dp ),
			'shipping_tax'              => wc_format_decimal( $order->get_shipping_tax(), $dp ),
			'total_discount'            => wc_format_decimal( $order->get_total_discount(), $dp ),
			'shipping_methods'          => $order->get_shipping_method(),
			'payment_details'           => array(
				'method_id'    => $order->get_payment_method(),
				'method_title' => $order->get_payment_method_title(),
				'paid'         => ! is_null( $order->get_date_paid() ),
			),
			'billing_address'           => array(
				'first_name' => $order->get_billing_first_name(),
				'last_name'  => $order->get_billing_last_name(),
				'company'    => $order->get_billing_company(),
				'address_1'  => $order->get_billing_address_1(),
				'address_2'  => $order->get_billing_address_2(),
				'city'       => $order->get_billing_city(),
				'state'      => $order->get_billing_state(),
				'postcode'   => $order->get_billing_postcode(),
				'country'    => $order->get_billing_country(),
				'email'      => $order->get_billing_email(),
				'phone'      => $order->get_billing_phone(),
			),
			'shipping_address'          => array(
				'first_name' => $order->get_shipping_first_name(),
				'last_name'  => $order->get_shipping_last_name(),
				'company'    => $order->get_shipping_company(),
				'address_1'  => $order->get_shipping_address_1(),
				'address_2'  => $order->get_shipping_address_2(),
				'city'       => $order->get_shipping_city(),
				'state'      => $order->get_shipping_state(),
				'postcode'   => $order->get_shipping_postcode(),
				'country'    => $order->get_shipping_country(),
			),
			'note'                      => $order->get_customer_note(),
			'customer_ip'               => $order->get_customer_ip_address(),
			'customer_user_agent'       => $order->get_customer_user_agent(),
			'customer_id'               => $order->get_user_id(),
			'view_order_url'            => $order->get_view_order_url(),
			'line_items'                => array(),
			'shipping_lines'            => array(),
			'tax_lines'                 => array(),
			'fee_lines'                 => array(),
			'coupon_lines'              => array(),
		);

		// add line items.
		foreach ( $order->get_items() as $item_id => $item ) {
			$product    = $item->get_product();
			$hideprefix = ( isset( $filter['all_item_meta'] ) && 'true' === $filter['all_item_meta'] ) ? null : '_';
			$item_meta  = $item->get_formatted_meta_data( $hideprefix );

			foreach ( $item_meta as $key => $values ) {
				$item_meta[ $key ]->label = $values->display_key;
				unset( $item_meta[ $key ]->display_key );
				unset( $item_meta[ $key ]->display_value );
			}

			$order_data['line_items'][] = array(
				'id'           => $item_id,
				'subtotal'     => wc_format_decimal( $order->get_line_subtotal( $item, false, false ), $dp ),
				'subtotal_tax' => wc_format_decimal( $item->get_subtotal_tax(), $dp ),
				'total'        => wc_format_decimal( $order->get_line_total( $item, false, false ), $dp ),
				'total_tax'    => wc_format_decimal( $item->get_total_tax(), $dp ),
				'price'        => wc_format_decimal( $order->get_item_total( $item, false, false ), $dp ),
				'quantity'     => $item->get_quantity(),
				'tax_class'    => $item->get_tax_class(),
				'name'         => $item->get_name(),
				'product_id'   => $item->get_variation_id() ? $item->get_variation_id() : $item->get_product_id(),
				'sku'          => is_object( $product ) ? $product->get_sku() : null,
				'meta'         => array_values( $item_meta ),
			);
		}

		// add shipping.
		foreach ( $order->get_shipping_methods() as $shipping_item_id => $shipping_item ) {
			$order_data['shipping_lines'][] = array(
				'id'           => $shipping_item_id,
				'method_id'    => $shipping_item->get_method_id(),
				'method_title' => $shipping_item->get_name(),
				'total'        => wc_format_decimal( $shipping_item->get_total(), $dp ),
			);
		}

		// add taxes.
		foreach ( $order->get_items( 'tax' ) as $tax_code => $tax ) {
			$order_data['tax_lines'][] = array(
				'id'       => $tax_code,
				'rate_id'  => $tax->get_rate_id(),
				'code'     => $tax->get_rate_code(),
				'title'    => $tax->get_label(),
				'total'    => wc_format_decimal( $tax->get_meta( 'tax_total' ), $dp ),
				'compound' => (bool) $tax->get_compound(),
			);
		}

		// add fees.
		foreach ( $order->get_fees() as $fee_item_id => $fee_item ) {
			$order_data['fee_lines'][] = array(
				'id'        => $fee_item_id,
				'title'     => $fee_item->get_name(),
				'tax_class' => $fee_item->get_tax_class(),
				'total'     => wc_format_decimal( $order->get_line_total( $fee_item ), $dp ),
				'total_tax' => wc_format_decimal( $order->get_line_tax( $fee_item ), $dp ),
			);
		}

		// add coupons.
		foreach ( $order->get_items( 'coupon' ) as $coupon_item_id => $coupon_item ) {
			$order_data['coupon_lines'][] = array(
				'id'     => $coupon_item_id,
				'code'   => $coupon_item->get_code(),
				'amount' => wc_format_decimal( $coupon_item->get_discount(), $dp ),
			);
		}

		return array( 'order' => apply_filters( 'woocommerce_api_order_response', $order_data, $order, $fields, null ) );
	}


	/**
	 * Format a unix timestamp or MySQL datetime into an RFC3339 datetime
	 *
	 * @since 2.1
	 * @param int|string $timestamp unix timestamp or MySQL datetime.
	 * @param bool       $convert_to_utc convert.
	 * @param bool       $convert_to_gmt Use GMT timezone.
	 * @return string RFC3339 datetime
	 */
	public function format_datetime( $timestamp, $convert_to_utc = false, $convert_to_gmt = false ) {
		if ( $convert_to_gmt ) {
			if ( is_numeric( $timestamp ) ) {
				$timestamp = date( 'Y-m-d H:i:s', $timestamp );
			}

			$timestamp = get_gmt_from_date( $timestamp );
		}

		if ( $convert_to_utc ) {
			$timezone = new DateTimeZone( wc_timezone_string() );
		} else {
			$timezone = new DateTimeZone( 'UTC' );
		}

		try {

			if ( is_numeric( $timestamp ) ) {
				$date = new DateTime( "@{$timestamp}" );
			} else {
				$date = new DateTime( $timestamp, $timezone );
			}

			// convert to UTC by adjusting the time based on the offset of the site's timezone.
			if ( $convert_to_utc ) {
				$date->modify( -1 * $date->getOffset() . ' seconds' );
			}
		} catch ( Exception $e ) {

			$date = new DateTime( '@0' );
		}

		return $date->format( 'Y-m-d\TH:i:s\Z' );
	}

	/**
	 * Permisos por usuario que de momento no usamos.
	 *
	 * @param string $post post.
	 *
	 * @return bool
	 */
	protected function is_readable( $post ) {
		return true;
	}

	/**
	 * Helper method to get order post objects
	 *
	 * @since 2.1
	 * @param array $args request arguments for filtering query.
	 * @return WP_Query
	 */
	protected function query_orders( $args ) {

		// set base query arguments.
		$statuses = wc_get_order_statuses();
		unset( $statuses['wc-pending'] );
		unset( $statuses['wc-completed'] );
		unset( $statuses['wc-cancelled'] );
		unset( $statuses['wc-refunded'] );
		unset( $statuses['wc-failed'] );
		$query_args = array(
			'fields'      => 'ids',
			'post_type'   => 'shop_order',
			'post_status' => array_keys( $statuses ),
		);

		// add status argument.
		if ( ! empty( $args['status'] ) ) {

			$statuses                  = 'wc-' . str_replace( ',', ',wc-', $args['status'] );
			$statuses                  = explode( ',', $statuses );
			$query_args['post_status'] = $statuses;

			unset( $args['status'] );
			unset( $statuses['wc-completed'] );
		}

		$query_args = $this->merge_query_args( $query_args, $args );

		// XImo y DANIEL: 30/06/2023
		// Vemos que está entregando de 10 en 10. Sucede que si tienes 10 pedidos futuros
		// Y no puedes procesarlas en glop y el estado de los pedidos de woocomerce
		// NO está cambiando y, por tanto, está siempre recibiendo los de hoy porque
		// Siempre le devuelve los mismos futuros y nunca los de hoy
		// Decidimos ampliar el posts_per_page a 100.
		$query_args['paged']          = false;
		$query_args['posts_per_page'] = 100;

		return new WP_Query( $query_args );
	}

	/**
	 * Add common request arguments to argument list before WP_Query is run
	 *
	 * @since 2.1
	 * @param array $base_args required arguments for the query (e.g. `post_type`, etc).
	 * @param array $request_args arguments provided in the request.
	 * @return array
	 */
	protected function merge_query_args( $base_args, $request_args ) {

		$args = array();

		// date.
		if ( ! empty( $request_args['created_at_min'] ) || ! empty( $request_args['created_at_max'] ) || ! empty( $request_args['updated_at_min'] ) || ! empty( $request_args['updated_at_max'] ) ) {

			$args['date_query'] = array();

			// resources created after specified date.
			if ( ! empty( $request_args['created_at_min'] ) ) {
				$args['date_query'][] = array(
					'column'    => 'post_date_gmt',
					'after'     => $request_args['created_at_min'],
					'inclusive' => true,
				);
			}

			// resources created before specified date.
			if ( ! empty( $request_args['created_at_max'] ) ) {
				$args['date_query'][] = array(
					'column'    => 'post_date_gmt',
					'before'    => $request_args['created_at_max'],
					'inclusive' => true,
				);
			}

			// resources updated after specified date.
			if ( ! empty( $request_args['updated_at_min'] ) ) {
				$args['date_query'][] = array(
					'column'    => 'post_modified_gmt',
					'after'     => $request_args['updated_at_min'],
					'inclusive' => true,
				);
			}

			// resources updated before specified date.
			if ( ! empty( $request_args['updated_at_max'] ) ) {
				$args['date_query'][] = array(
					'column'    => 'post_modified_gmt',
					'before'    => $request_args['updated_at_max'],
					'inclusive' => true,
				);
			}
		}

		// search.
		if ( ! empty( $request_args['q'] ) ) {
			$args['s'] = $request_args['q'];
		}

		// resources per response.
		if ( ! empty( $request_args['limit'] ) ) {
			$args['posts_per_page'] = $request_args['limit'];
		}

		// resource offset.
		if ( ! empty( $request_args['offset'] ) ) {
			$args['offset'] = $request_args['offset'];
		}

		// order (ASC or DESC, ASC by default).
		if ( ! empty( $request_args['order'] ) ) {
			$args['order'] = $request_args['order'];
		}

		// orderby.
		if ( ! empty( $request_args['orderby'] ) ) {
			$args['orderby'] = $request_args['orderby'];

			// allow sorting by meta value.
			if ( ! empty( $request_args['orderby_meta_key'] ) ) {
				$args['meta_key'] = $request_args['orderby_meta_key'];
			}
		}

		// allow post status change.
		if ( ! empty( $request_args['post_status'] ) ) {
			$args['post_status'] = $request_args['post_status'];
			unset( $request_args['post_status'] );
		}

		// filter by a list of post id.
		if ( ! empty( $request_args['in'] ) ) {
			$args['post__in'] = explode( ',', $request_args['in'] );
			unset( $request_args['in'] );
		}

		// filter by a list of post id.
		if ( ! empty( $request_args['in'] ) ) {
			$args['post__in'] = explode( ',', $request_args['in'] );
			unset( $request_args['in'] );
		}

		// resource page.
		$args['paged'] = ( isset( $request_args['page'] ) ) ? absint( $request_args['page'] ) : 1;

		$args = apply_filters( 'woocommerce_api_query_args', $args, $request_args );

		return array_merge( $base_args, $args );
	}

	/**
	 * @param array      $line_item line item.
	 * @param WC_Product $p         Producto.
	 */
	private function extract_articulos_extras( $line_item, $p ) {
		$metas = array();
		if ( '62.57.20.247' === getenv( 'REMOTE_ADDR' ) ) {
//			echo "<textarea>";print_r($p);echo "</textarea>";
//			echo "<textarea>";print_r($line_item);echo "</textarea>";exit();
		}
		foreach ( $p as $tipo_obj ) {
			$nombre_objeto      = sanitize_title( $tipo_obj['name'] );
			$is_articulos_extra = false !== strpos( 'articulos-extras', $nombre_objeto );
			if ( 's' === (string) $tipo_obj['is_menu'] ) {
				if ( ! isset( $metas['menus'] ) ) {
					$metas['menus'] = array();
				}
				foreach ( $line_item['meta'] as $item ) {
					if ( (string) $item->key === (string) $tipo_obj['name'] ) {
						$value = $item->value;
						// BUSCAR LA OPCIÓN DEL ADDON QUE TENGA ESTE VALOR PARA OBTENER PRECIO Y CODIGO.
						foreach ( $tipo_obj['options'] as $option ) {
							$from                    = '/'.preg_quote( ' - ', '/').'/';
							$value                   = preg_replace( $from, '|', $value, 1 );
							list( $codigo, $nombre ) = explode( '|', $value );
							if (
								( empty( $nombre ) && $option['label'] === $value ) ||
								$option['codigo'] === $codigo
							) {
								$arra = array(
									'etiqueta' => $option['label'],
									'codigo'   => $option['codigo'],
									'grupo'    => $tipo_obj['name'],
								);
								if ( ! empty( $option['price'] ) ) {
									$arra['precio'] = floatval( $option['price'] ) * 100;
								}
								$metas['menus'][] = $arra;
							}
						}

//						get_option( 'grupo_' . $opcion['codigo'] )
					}
				}
			} else {
				foreach ( $line_item['meta'] as $item ) {
					if (
						(string) $item->key === (string) $tipo_obj['name'] ||
						( $is_articulos_extra && false !== strpos( sanitize_title( $item->key ), $nombre_objeto ) )
					) {
						// SALSA BARBACOA DEL ITEM.
						$value                   = $item->value;
						list( $codigo, $nombre ) = explode( ' - ', $value );
						foreach ( $tipo_obj['options'] as $option ) {

							if ( $option['label'] === $value || $codigo === $option['codigo'] ) {
								if ( ! array_key_exists( $nombre_objeto, $metas ) ) {
									$metas[ $nombre_objeto ] = array();
								}
								$arra = array(
									'etiqueta' => $option['label'],
									'codigo'   => $option['codigo'],
								);
								if ( ! empty( $option['price'] ) ) {
									$arra['precio'] = floatval( $option['price'] ) * 100;
								}
								$metas[ $nombre_objeto ][] = $arra;
							}
						}
					}
				}
			}
		}
		return $metas;

	}

	private function extract_combinados( array $line_item, $product_addons ): array {
		$metas = array();

		foreach ( $product_addons as $addon ) {
			if (
				isset( $addon['belongsToVariation'] )
			) {
				// Es combinado.

			}
		}

		return $metas;
	}
}
