<?php
/**
 * REST API: Wooglop_REST_Glop_Store_Controller class
 *
 * @package Glop
 * @subpackage wooglop
 * @since 1.0.0
 */

/**
 * Core class to GLOP via the REST API.
 *
 * @since 1.0.0
 *
 * @see Wooglop_WP_REST_Controller
 */
class Wooglop_REST_Glop_Store_Controller extends Wooglop_WP_REST_Controller {

	/**
	 * Registers the routes for the objects of the controller.
	 *
	 * @since 1.0.0
	 *
	 * @see register_rest_route()
	 */
	public function register_routes() {
		$this->rest_base = 'store';
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				'args'   => array(),
				array(
					'methods'             => WP_REST_Server::EDITABLE,
					'callback'            => array( $this, 'update_store_options' ),
					'permission_callback' => array( $this, 'get_items_permissions_check_no_key' ),
					'args'                => $this->get_endpoint_args_for_item_schema( WP_REST_Server::EDITABLE ),
				),
				'schema' => array( $this, 'get_public_item_schema' ),
			)
		);
	}

    /**
     * Validates a field from glop.
     *
     * @param mixed $field The field to validate.
     * @return bool Returns true if the field is set, false otherwise.
     */
    private function validate_field_from_glop( $field ): bool {
        return isset( $field ) && $field !== '%&';
    }

	/**
	 * Crear JSON object
	 *
	 * @since 1.0.0
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function update_store_options( $request ) {
		try {
			$this->log_request( $request );
			$options = $request->get_param( 'options' );

			// DANIEL RUIZ: 03 Agosto 2021 #2331
			$options = array_change_key_case( $options, CASE_LOWER );

			if ( isset( $options['aviso'] ) ) {
				update_option( 'woocommerce_demo_store',
					0 === intval( $options['aviso'] ) ? 'no' : 'yes' );
			}

			if ( isset( $options['aviso_mensaje'] ) ) {
				update_option( 'woocommerce_demo_store_notice', esc_html( $options['aviso_mensaje'] ) );
			}

			if ( isset( $options['cerrar_tienda'] ) ) {
				update_option( 'byconsolewooodt_cerrar_tienda',
					1 === intval( $options['cerrar_tienda'] ) ? 'on' : '' );
			}

			if ( isset( $options['delivery_time'] ) ) {
				update_option( 'byconsolewooodt_chekout_page_section_heading', esc_html( $options['delivery_time'] ) );
			}

			if ( isset( $options['caracteres_descripcion_corta'] ) ) {
				update_option( 'wglop_n_caracteres_cortos', absint( $options['caracteres_descripcion_corta'] ) );
			}

			if ( isset( $options['texto_boton_add_to_cart'] ) ) {
				update_option( 'wglop_texto_boton_add_to_cart', esc_html( $options['texto_boton_add_to_cart'] ) );
			}

			if ( isset( $options['texto_boton_finalize_order'] ) ) {
				update_option( 'wglop_texto_boton_finalizar_compra', esc_html( $options['texto_boton_finalize_order'] ) );
			}

			if ( isset( $options['texto_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_seccion_top_ventas', esc_html( $options['texto_seccion_top_ventas'] ) );
			}

			if ( isset( $options['descripcion_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_descripcion_top_ventas', esc_html( $options['descripcion_seccion_top_ventas'] ) );
			}

			if( isset( $options['cw_texto_header_titulo'] ) ) {
				update_option( 'wglop_header_titulo', $options['cw_texto_header_titulo'] );
			}

			if( isset( $options['cw_texto_header_descripcion'] ) ) {
				update_option( 'wglop_header_descripcion', $options['cw_texto_header_descripcion'] );
			}

			if( isset( $options['cw_texto_boton_pedir_comida'] ) ) {
				update_option( 'wglop_header_texto_pide_comida', $options['cw_texto_boton_pedir_comida'] );
			}

			if( isset( $options['cw_texto_boton_finalizar_compra'] ) ) {
				update_option( 'wglop_header_texto_finalizar_compra', $options['cw_texto_boton_finalizar_compra'] );
			}

			if( isset( $options['cw_texto_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_seccion_top_ventas', $options['cw_texto_seccion_top_ventas'] );
			}

			if( isset( $options['cw_descr_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_descripcion_top_ventas', $options['cw_descr_seccion_top_ventas'] );
			}

			if( isset( $options['cw_caracteres_descripcion_corta'] ) ) {
				update_option( 'wglop_n_caracteres_cortos', $options['cw_caracteres_descripcion_corta'] );
			}

			if( isset( $options['cw_texto_boton_login'] ) ) {
				update_option( 'wglop_texto_boton_login', $options['cw_texto_boton_login'] );
			}

			if( isset( $options['cw_texto_boton_add_cart_modal'] ) ) {
				update_option( 'wglop_texto_boton_add_to_cart', $options['cw_texto_boton_add_cart_modal'] );
			}

			if( isset( $options['cw_texto_boton_final_compra'] ) ) {
				update_option( 'wglop_texto_boton_finalizar_compra', $options['cw_texto_boton_final_compra'] );
			}

			if( isset( $options['cw_accent_color'] ) ) {
				set_theme_mod( 'accent_color', $options['cw_accent_color'] );
			}

			if( isset( $options['cw_accent_color_deliver_type'] ) ) {
				set_theme_mod( 'accent_color_deliver_type', $options['cw_accent_color_deliver_type'] );
			}

			if( isset( $options['cw_font_color_deliver_type'] ) ) {
				set_theme_mod( 'font_color_deliver_type', $options['cw_font_color_deliver_type'] );
			}

            // Campos nuevos para delivery.glop.es.
			if( isset( $options['cw_header_titulo_color'] ) ) {
				set_theme_mod( 'header_titulo_color', $options['cw_header_titulo_color'] );
			}

			if( isset( $options['cw_wglop_header_titulo_text_size'] ) ) {
				update_option( 'wglop_header_titulo_text_size', $options['cw_wglop_header_titulo_text_size'] );
			}

			if( isset( $options['cw_wglop_header_color_fondo'] ) ) {
                set_theme_mod( 'wglop_header_color_fondo', $options['cw_wglop_header_color_fondo'] );
			}

			if( isset( $options['cw_wglop_footer_color_fondo'] ) ) {
                set_theme_mod( 'wglop_footer_color_fondo', $options['cw_wglop_footer_color_fondo'] );
			}

			if( isset( $options['cw_wglop_footer_color_letra'] ) ) {
                set_theme_mod( 'wglop_footer_color_letra', $options['cw_wglop_footer_color_letra'] );
			}

			if( isset( $options['cw_wglop_productos_color_fondo'] ) ) {
                set_theme_mod( 'wglop_productos_color_fondo', $options['cw_wglop_productos_color_fondo'] );
			}

			if( isset( $options['cw_wglop_productos_color_texto'] ) ) {
                set_theme_mod( 'wglop_productos_color_texto', $options['cw_wglop_productos_color_texto'] );
			}

			if( isset( $options['cw_wglop_titulos_color_texto'] ) ) {
                set_theme_mod( 'wglop_titulos_color_texto', $options['cw_wglop_titulos_color_texto'] );
			}

			if( isset( $options['cw_wglop_btn_entrega_c_texto'] ) ) {
                set_theme_mod( 'wglop_btn_entrega_c_texto', $options['cw_wglop_btn_entrega_c_texto'] );
			}

			if( isset( $options['cw_wglop_botones_color_texto'] ) ) {
                set_theme_mod( 'wglop_botones_color_texto', $options['cw_wglop_botones_color_texto'] );
			}

			if( $this->validate_field_from_glop( $options['cw_texto_destacados'] ) ) {
                set_theme_mod( 'wglop_cw_texto_destacados', $options['cw_texto_destacados'] );
			}
            if( $this->validate_field_from_glop( $options['cw_texto_sitio'] ) ) {
                update_option( 'blogname', $options['cw_texto_sitio']);
            }

            if( $this->validate_field_from_glop( $options['cw_texto_sitio_t'] ) ) {
                set_theme_mod( 'wglop_cw_texto_sitio_t', $options['cw_texto_sitio_t']);
            }

            if( $this->validate_field_from_glop( $options['cw_boton_fin_compra'] ) ) {
                set_theme_mod( 'wglop_cw_boton_fin_compra', $options['cw_boton_fin_compra']);
            }

            if( $this->validate_field_from_glop( $options['cw_texto_boton_para_llevar'] ) ) {
                set_theme_mod( 'wglop_cw_texto_boton_para_llevar', $options['cw_texto_boton_para_llevar']);
            }

            if( $this->validate_field_from_glop( $options['cw_texto_boton_recoger_en_local'] ) ) {
                set_theme_mod( 'wglop_cw_texto_boton_recoger_en_local', $options['cw_texto_boton_recoger_en_local']);
            }

            if( $this->validate_field_from_glop( $options['cw_texto_boton_recoger_en_est'] ) ) {
                set_theme_mod( 'wglop_cw_texto_boton_recoger_en_est', $options['cw_texto_boton_recoger_en_est']);
            }

            if( $this->validate_field_from_glop( $options['cw_tipo_entrega_addons'] ) ) {
                set_theme_mod( 'wglop_cw_tipo_entrega_addons', $options['cw_tipo_entrega_addons']);
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_clave_secreta'] ) ) {
                set_theme_mod( 'wglop_cw_pidepaga_clave_secreta', $options['cw_pidepaga_clave_secreta']);
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_minutos_bloqueada'] ) ) {
                update_option( 'glop_pideypaga_minutos_desbloqueo', $options['cw_pidepaga_minutos_bloqueada']);
            } else {
                update_option( 'glop_pideypaga_minutos_desbloqueo', 1 );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_mensaje_bloqueo'] ) ) {
                update_option( 'glop_pideypaga_mensaje_desbloqueo', $options['cw_pidepaga_mensaje_bloqueo']);
            } else {
                update_option( 'glop_pideypaga_mensaje_desbloqueo', __( 'La cuenta se encuentra bloqueada durante %s', 'wooglop-pidepaga' ));
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_menaje_popup'] ) ) {
                update_option( 'glop_pideypaga_mensaje_popup', $options['cw_pidepaga_menaje_popup'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_boton_pedir'] ) ) {
                update_option( 'glop_pideypaga_boton_pedir', $options['cw_pidepaga_boton_pedir'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_boton_pagar'] ) ) {
                update_option( 'glop_pideypaga_boton_pagar', $options['cw_pidepaga_boton_pagar'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_boton_confirmac'] ) ) {
                update_option( 'glop_pideypaga_boton_vale', $options['cw_pidepaga_boton_confirmac']);
            } else {
                update_option( 'glop_pideypaga_boton_vale', 'Vale' );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_pagar_fondo'] ) ) {
                update_option( 'glop_pideypaga_boton_pagar_background', $options['cw_pidepaga_pagar_fondo'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_pagar_letra'] ) ) {
                update_option( 'glop_pideypaga_boton_pagar_letra', $options['cw_pidepaga_pagar_letra'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_pedir_fondo'] ) ) {
                update_option( 'glop_pideypaga_boton_pedir_background', $options['cw_pidepaga_pedir_fondo'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_pedir_letra'] ) ) {
                update_option( 'glop_pideypaga_boton_pedir_letra', $options['cw_pidepaga_pedir_letra'] );
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_conf_fondo'] ) ) {
                update_option( 'glop_pideypaga_boton_ok_background', $options['cw_pidepaga_conf_fondo']);
            }

            if( $this->validate_field_from_glop( $options['cw_pidepaga_conf_letra'] ) ) {
                update_option( 'glop_pideypaga_boton_ok_letra', $options['cw_pidepaga_conf_letra']);
            }

            if( $this->validate_field_from_glop( $options['cw_productos_3_columnas'] ) ) {
                set_theme_mod( 'wglop_cw_productos_3_columnas', $options['cw_productos_3_columnas']);
            }


            // INPUT TEXTBOXES
            if( $this->validate_field_from_glop( $options['cw_addon_diasdeprepedido'] ) ) {
                update_option( 'byconsolewooodt_preorder_days', $options['cw_addon_diasdeprepedido']);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_horariodeapertura1'] ) ) {
                update_option( 'byconsolewooodt_opening_hours_from', $options['cw_addon_horariodeapertura1']);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_horariodeapertura2'] ) ) {
                update_option( 'byconsolewooodt_opening_hours_to', $options['cw_addon_horariodeapertura2']);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_fechadeentrega1'] ) ) {
                update_option( 'byconsolewooodt_delivery_hours_from', $options['cw_addon_fechadeentrega1']);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_fechadeentrega2'] ) ) {
                update_option( 'byconsolewooodt_delivery_hours_to', $options['cw_addon_fechadeentrega2']);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_tipodeentregadelpedido'] ) ) {
                update_option( 'cw_addon_tipodeentregadelpedido', $options['cw_addon_tipodeentregadelpedido']);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_fechaehoraopcional'] ) ) {
                $array = ['Oculto', 'Opcional', 'Obligatorio'];
                update_option( 'byconsolewooodt_hours_format_optional', $array[ intval($options['cw_addon_fechaehoraopcional'] ) ] );
            }
            if( $this->validate_field_from_glop( $options['cw_addons_campo_pedido_establ'] ) ) {
                $array = ['Oculto', 'Opcional', 'Obligatorio'];
                update_option( 'byconsolewooodt_chekout_page_alias_texto_opcional', $array[ intval($options['cw_addons_campo_pedido_establ']) ]);
            }
            if( $this->validate_field_from_glop( $options['cw_addon_formatodetiempo'] ) ) {
                $array = ['24 horas', '24 horas (sin segundos)', '12 horas'];
                update_option( 'byconsolewooodt_hours_format', $array[ intval($options['cw_addon_formatodetiempo']) ]);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_tiempos_entrega_mini'] ) ) {
                update_option( 'byconsolewooodt_delivery_times', $options['cw_addons_tiempos_entrega_mini']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_pedido_sera_entregad'] ) ) {
                update_option( 'byconsolewooodt_orders_delivered', $options['cw_addons_pedido_sera_entregad']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_pedido_sera_recogido'] ) ) {
                update_option( 'byconsolewooodt_orders_pick_up', $options['cw_addons_pedido_sera_recogido']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_etiqueta_de_recogida'] ) ) {
                update_option( 'byconsolewooodt_takeaway_lable', $options['cw_addons_etiqueta_de_recogida']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_etiqueta_de_entrega'] ) ) {
                update_option( 'byconsolewooodt_delivery_lable', $options['cw_addons_etiqueta_de_entrega']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_etiqueta_de_establec'] ) ) {
                update_option( 'byconsolewooodt_establecimiento_lable', $options['cw_addons_etiqueta_de_establec']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_campo_de_fecha'] ) ) {
                update_option( 'byconsolewooodt_date_field_text', $options['cw_addons_campo_de_fecha']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_campo_de_hora'] ) ) {
                update_option( 'byconsolewooodt_time_field_text', $options['cw_addons_campo_de_hora']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_a_mostrar_en_la_pag'] ) ) {
                update_option( 'byconsolewooodt_chekout_page_section_heading', $options['cw_addons_a_mostrar_en_la_pag']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_tipo_de_pedido_pago'] ) ) {
                update_option( 'byconsolewooodt_chekout_page_order_type_label', $options['cw_addons_tipo_de_pedido_pago']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_calendario_en_pago'] ) ) {
                update_option( 'byconsolewooodt_chekout_page_date_label', $options['cw_addons_calendario_en_pago']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_seleccion_de_tiempo'] ) ) {
                update_option( 'byconsolewooodt_chekout_page_time_label', $options['cw_addons_seleccion_de_tiempo']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_etiqueta_para_selec'] ) ) {
                update_option( 'byconsolewooodt_chekout_page_alias_texto', $options['cw_addons_etiqueta_para_selec']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_placeholder_de_la_e'] ) ) {
                update_option( 'byconsolewooodt_chekout_page_alias_placeholder', $options['cw_addons_placeholder_de_la_e']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_email_al_que_se_env'] ) ) {
                update_option( 'byconsolewooodt_chekout_email_pedido_establecimiento', $options['cw_addons_email_al_que_se_env']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_nombre_al_que_se_en'] ) ) {
                update_option( 'byconsolewooodt_chekout_nombre_pedido_establecimiento', $options['cw_addons_nombre_al_que_se_en']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_apellidos_al_que_se'] ) ) {
                update_option( 'byconsolewooodt_chekout_apellidos_pedido_establecimiento', $options['cw_addons_apellidos_al_que_se']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_texto_sobre_su_turn'] ) ) {
                update_option( 'byconsolewooodt_chekout_texto_su_turno', $options['cw_addons_texto_sobre_su_turn']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_textoarticulos'] ) ) {
                update_option( 'byconsolewooodt_chekout_articulos_extra_type_label', $options['cw_addons_textoarticulos']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_textocomentarios'] ) ) {
                update_option( 'byconsolewooodt_chekout_comentarios_libres_type_label', $options['cw_addons_textocomentarios']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_textocombinacon'] ) ) {
                update_option( 'byconsolewooodt_chekout_combina_con_type_label', $options['cw_addons_textocombinacon']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_textosinreparto'] ) ) {
                update_option( 'byconsolewooodt_chekout_no_disponible_pedido_type_label', $options['cw_addons_textosinreparto']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_textosinrepartoboton'] ) ) {
                update_option( 'byconsolewooodt_chekout_no_disponible_pedido_finalizar_type_label', $options['cw_addons_textosinrepartoboton']);
            }
            if( $this->validate_field_from_glop( $options['cw_addons_textocabeceraenvio'] ) ) {
                update_option( 'byconsolewooodt_detalels_del_envio_label', $options['cw_addons_textocabeceraenvio']);
            }
            if( $this->validate_field_from_glop( $options['cw_pidepaga_pago_nombre'] ) ) {
                $pidepagasettings = get_option('woocommerce_pidepaga_settings');
                $pidepagasettings['title'] = $options['cw_pidepaga_pago_nombre'];
                update_option( 'woocommerce_pidepaga_settings', $pidepagasettings );
            }
            if( $this->validate_field_from_glop( $options['cw_pidepaga_pago_descri'] ) ) {
                $pidepagasettings = get_option('woocommerce_pidepaga_settings');
                $pidepagasettings['description'] = $options['cw_pidepaga_pago_descri'];
                update_option( 'woocommerce_pidepaga_settings', $pidepagasettings );
            }

            if( 0 < strlen(  $options['cw_pidepaga_monei22'] ) ) {
                $apirest = new Wooglop_Pidepaga_Api_Rest_Connector();
                $apirest->save_monei_pklive( $options['cw_pidepaga_monei'] );
                update_option( 'glop_pideypaga_pklive', $options['cw_pidepaga_monei'] );
            }








            // CHECK BOXES
            if ( isset( $options['mostrar_imagenes'] ) ) {
                update_option( 'wglop_ocultar_imagen_producto', 1 === intval( $options['mostrar_imagenes'] ) ? 'no' : 'yes' );
            }

            if ( isset( $options['top_ventas_enabled'] ) ) {
                update_option( 'wglop_top_ventas_enabled', 1 === intval( $options['top_ventas_enabled'] ) ? 'yes' : 'no' );
            }
            if ( isset( $options['orden_categorias'] ) ) {
                $categorias = $options['orden_categorias'];
                $this->reordenar_categorias( $categorias );
            }

            if ( isset( $options['top_ventas_products'] ) ) {
                $productos = $options['top_ventas_products'];
                $this->set_top_ventas( $productos );
            }

            if ( isset( $options['alergenos'] ) ) {
                $alergenos = $options['alergenos'];
                $this->set_alergenos( $alergenos );
            }

            if( isset( $options['cw_ocultar_imagen_producto'] ) ) {
                update_option( 'wglop_ocultar_imagen_producto', 1 === intval( $options['cw_ocultar_imagen_producto'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_activar_top_ventas'] ) ) {
                update_option( 'wglop_top_ventas_enabled', 1 === intval( $options['cw_activar_top_ventas'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_cambiar_icono_idiomas'] ) ) {
                update_option( 'wglop_languages_enabled', 1 === intval( $options['cw_cambiar_icono_idiomas'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_activar_vista_minimalista'] ) ) {
                update_option( 'wglop_minimalista_enabled', 1 === intval( $options['cw_activar_vista_minimalista'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_mostrar_descripcion_corta'] ) ) {
                update_option( 'wglop_minimalista_mostrar_descripcion_corta_enabled', 1 === intval( $options['cw_mostrar_descripcion_corta'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_mostrar_alergenos'] ) ) {
                update_option( 'wglop_minimalista_mostrar_alergenos_enabled', 1 === intval( $options['cw_mostrar_alergenos'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_mostrar_detalle_articulos'] ) ) {
                update_option( 'wglop_minimalista_ocultar_detalles_articulos_enabled', 1 === intval( $options['cw_mostrar_detalle_articulos'] ) ? 'no' : 'yes' );
            }

            if ( isset( $options['cw_wglop_is_una_columna'] ) ) {
                update_option( 'wooglop_is_una_columna', 1 === intval( $options['cw_wglop_is_una_columna'] ) ? 'yes' : 'no' );
            }

            if ( isset( $options['cw_productos_3_columnas'] ) ) {
                update_option( 'wooglop_productos_a_tres_columnas', 1 === intval( $options['cw_productos_3_columnas'] ) ? 'yes' : 'no' );
            }

            if( isset( $options['cw_addon_activardelivery'] ) ) {
                update_option( 'byconsolewooodt_enabled', 1 === intval( $options['cw_addon_activardelivery'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_ocultarmenus'] ) ) {
                update_option( 'byconsolewooodt_disabled_menus', 1 === intval( $options['cw_addon_ocultarmenus'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_cerrartienda'] ) ) {
                // Cerrar tienda se establece desde otra zona de glop y no desde Configuraciones Web
//                update_option( 'byconsolewooodt_cerrar_tienda', 1 === intval( $options['cw_addon_cerrartienda'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_obligatorioseleccionm'] ) ) {
                update_option( 'byconsolewooodt_required_menus', 1 === intval( $options['cw_addon_obligatorioseleccionm'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_ocultaringredientesex'] ) ) {
                update_option( 'byconsolewooodt_disabled_ingredientes', 1 === intval( $options['cw_addon_ocultaringredientesex'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_ocultarcomentarios'] ) ) {
                update_option( 'byconsolewooodt_disabled_comentarios', 1 === intval( $options['cw_addon_ocultarcomentarios'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_forzarventaunitaria'] ) ) {
                update_option( 'byconsolewooodt_only_cantidad_unitaria', 1 === intval( $options['cw_addon_forzarventaunitaria'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addon_activarsuturno'] ) ) {
                update_option( 'byconsolewooodt_chekout_enable_su_turno', 1 === intval( $options['cw_addon_activarsuturno'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addons_clientenotaspedido'] ) ) {
                update_option( 'byconsolewooodt_hay_notas_label', 1 === intval( $options['cw_addons_clientenotaspedido'] ) ? 'on' : '');
            }
            if( isset( $options['cw_addons_direccionenvio'] ) ) {
                update_option( 'byconsolewooodt_permitir_envio_diferente_label', 1 === intval( $options['cw_addons_direccionenvio'] ) ? 'on' : '');
            }

            if( isset( $options['cw_addons_activar_pidepaga'] ) ) {
                update_option( 'glop_activar_pideypaga',
                    1 === intval( $options['cw_addons_activar_pidepaga'] ) ? 'yes' : ''
                );
            }

            if( isset( $options['cw_wooglop_modo_catalogo'] ) ) {
                update_option( 'cw_wooglop_modo_catalogo',
                    1 === intval( $options['cw_wooglop_modo_catalogo'] ) ? 'yes' : ''
                );
            }


			if( isset( $options['cw_mostrar_todos_los_productos'] ) ) {
                update_option( 'wglop_mostrar_todos_los_productos', 1 === intval( $options['cw_mostrar_todos_los_productos'] ) ? 'yes' : 'no' );
                set_theme_mod( 'wglop_botones_color_texto', 1 === intval( $options['cw_mostrar_todos_los_productos'] ) ? 'yes' : 'no' );
                // Al activar mostrar todos los productos en portada → deberemos activar el checkbox top ventas tanto en glop como en la plantilla.
                if ( 1 === intval( $options['cw_mostrar_todos_los_productos'] ) ) {
                    update_option( 'wglop_top_ventas_enabled', 'yes' );
                }
			}

			if( isset( $options['cw_wglop_logo'] ) ) {
                if ( ! empty( $options['cw_wglop_logo']['src'] ) ) {
                    $upload        = Wooglop_Product::upload_product_image( $options['cw_wglop_logo'] );
                    $attachment_id = Wooglop_Product::set_product_image_as_attachment_static( $upload );
                    set_theme_mod( 'custom_logo', $attachment_id );
                } else {
                    set_theme_mod( 'custom_logo', null );
                }
			}

			if( isset( $options['cw_wglop_header_fondo_img'] ) ) {
                // Obtener página id de portada y vincularle el thumbnail.
                $front_page_id = get_option( 'page_on_front' );
                if ( ! empty( $options['cw_wglop_header_fondo_img']['src'] ) ) {
                    $upload        = Wooglop_Product::upload_product_image( $options['cw_wglop_header_fondo_img'] );
                    $attachment_id = Wooglop_Product::set_product_image_as_attachment_static( $upload );
                    set_post_thumbnail( $front_page_id, $attachment_id );
                } else {
                    delete_post_thumbnail( $front_page_id );
                }
			}

			if( isset( $options['cw_borrar_todo'] ) && 1 === intval( $options['cw_borrar_todo'] ) ) {
				// BORRAR CATEGORÍAS Y PRODUCTOS
				$allposts= get_posts( array('post_type'=>'product','numberposts'=>-1) );
				foreach ($allposts as $eachpost) {
					wp_delete_post( $eachpost->ID, true );
				}

				$taxonomy_name = 'product_cat';
				$terms = get_terms( array(
					'taxonomy' => $taxonomy_name,
					'hide_empty' => false
				) );
				foreach ( $terms as $term ) {
					wp_delete_term($term->term_id, $taxonomy_name);
				}
			}

		}catch (Exception $e) {
			if(getenv("REMOTE_ADDR") == '62.57.20.247') {
				echo "<textarea>";print_r($e);echo "</textarea>";exit();
			}
		}
		return rest_ensure_response( array() );

	}

	/**
	 * SKU de productos.
	 *
	 * @param array $alergenos Productos.
	 */
	private function set_alergenos( $alergenos ) {
		if ( ! is_array( $alergenos ) || 0 === count( $alergenos ) ) {
			return;
		}

		$alergenos_options = array();

		// #2945 19/04/2022 Daniel Ruiz.
		$alergenos_borrar = get_option( 'wglop_alergenos', null );
		if ( ! empty( $alergenos_borrar ) ) {
			foreach ( $alergenos_borrar as $alergeno ) {
				if ( isset( $alergeno['file'] ) ) {
					@unlink( $alergeno['file'] );
				}
			}
		}

		foreach ( $alergenos as $alergeno ) {
			// guardar file en uploads.
			if ( $alergeno['src'] ) {
				$upload = Wooglop_Product::upload_product_image( $alergeno );
				$alergenos_options[ sanitize_title( $alergeno['nombre'] ) ] = $upload;
			}
		}

		update_option( 'wglop_alergenos', $alergenos_options );
	}

	/**
	 * SKU de productos.
	 *
	 * @param array $productos Productos.
	 */
	private function set_top_ventas( $productos ) {
		/**
		 * Producto.
		 *
		 * @var WC_Product[] $products Product.
		 */
		$products = wc_get_products(
			array(
				'featured'   => true,
				'visibility' => 'featured',
			)
		);

		if ( isset( $products ) && 0 < count( $products ) ) {
			foreach ( $products as $product ) {
				$product->set_featured( false );
				$product->save();
			}
		}

		global $wpdb;

		$menu_orders = wp_list_pluck(
			$wpdb->get_results(
				"SELECT ID, menu_order FROM {$wpdb->posts} WHERE post_type = 'product' ORDER BY menu_order ASC, post_title ASC"
			),
			'menu_order',
			'ID'
		);
		$_pf         = new WC_Product_Factory();
		foreach ( $productos as $key => $producto ) {
			$producto = $productos[ $key ];
			$product  = wc_get_product_id_by_sku( $producto );

			if ( ! isset( $product ) ) {
				continue;
			}
			/**
			 * Producto.
			 *
			 * @var WC_Product $product Product.
			 */
			$producto = $_pf->get_product( $product );
			unset( $menu_orders[ $producto->get_id() ] );
		}

		foreach ( $productos as $key => $producto ) {
			$producto = $productos[ $key ];
			$product  = wc_get_product_id_by_sku( $producto );

			if ( ! isset( $product ) ) {
				continue;
			}
			/**
			 * Producto.
			 *
			 * @var WC_Product $product Product.
			 */
			$producto = $_pf->get_product( $product );
			$producto->set_featured( true );
			$producto->save();
			$args['id'] = $producto->get_id();
			if ( 0 < $key ) {
				$product_prev = wc_get_product_id_by_sku( $productos[ $key - 1 ] );
				if ( ! isset( $product_prev ) ) {
					continue;
				}
				$args['previd'] = $product_prev;
			}
			if ( count( $productos ) - 1 === $key ) {
				$first          = array_keys( $menu_orders );
				$args['nextid'] = current( $first );
			} else {
				$product_next = wc_get_product_id_by_sku( $productos[ $key + 1 ] );
				if ( ! isset( $product_next ) ) {
					continue;
				}
				if ( 0 < $key ) {
					$args['nextid'] = $product_next;
				} else {
					$first = array_keys( $menu_orders );
					reset( $first );
					$args['nextid'] = current( $first );
				}
			}

			$this->wooglop_product_ordering( $args );

		}
	}

	/**
	 *
	 * Ordenar productos.
	 *
	 * @param array $args Argumentos.
	 */
	private function wooglop_product_ordering( $args ) {
		// phpcs:disable
		global $wpdb;
		$sorting_id  = absint( $args['id'] );
		$previd      = absint( isset( $args['previd'] ) ? $args['previd'] : 0 );
		$nextid      = absint( isset( $args['nextid'] ) ? $args['nextid'] : 0 );

		$menu_orders = wp_list_pluck( $wpdb->get_results( "SELECT ID, menu_order FROM {$wpdb->posts} WHERE post_type = 'product' ORDER BY menu_order ASC, post_title ASC" ), 'menu_order', 'ID' );
		$index       = 0;

		foreach ( $menu_orders as $id => $menu_order ) {
			$id = absint( $id );

			if ( $sorting_id === $id ) {
				continue;
			}
			if ( $nextid === $id ) {
				$index ++;
			}
			$index ++;
			$menu_orders[ $id ] = $index;
			$wpdb->update( $wpdb->posts, array( 'menu_order' => $index ), array( 'ID' => $id ) );

			/**
			 * When a single product has gotten it's ordering updated.
			 * $id The product ID
			 * $index The new menu order
			 */
			do_action( 'woocommerce_after_single_product_ordering', $id, $index );
		}

		if ( isset( $menu_orders[ $previd ] ) ) {
			$menu_orders[ $sorting_id ] = $menu_orders[ $previd ] + 1;
		} elseif ( isset( $menu_orders[ $nextid ] ) ) {
			$menu_orders[ $sorting_id ] = $menu_orders[ $nextid ] - 1;
		} else {
			$menu_orders[ $sorting_id ] = 0;
		}

		$wpdb->update( $wpdb->posts, array( 'menu_order' => $menu_orders[ $sorting_id ] ), array( 'ID' => $sorting_id ) );

		WC_Post_Data::delete_product_query_transients();

		do_action( 'woocommerce_after_product_ordering', $sorting_id, $menu_orders );
		// phpcs:enable
	}

	/**
	 * Get admin user id.
	 *
	 * @return int
	 */
	private function wooglop_admin_user_id() {
		global $wpdb;

		$wp_user_search = $wpdb->get_results( "SELECT ID, display_name FROM $wpdb->users ORDER BY ID" );
		foreach ( $wp_user_search as $userid ) {
			$curuser    = get_userdata( $userid->ID );
			$user_level = $curuser->user_level;
			if ( 8 <= $user_level ) {
				return $userid->ID;
			}
		}
		return null;
	}

	/**
	 * Reordenar categorias.
	 *
	 * @param Array $categorias Categorias.
	 * @return mixed|WP_Error|WP_HTTP_Response|WP_REST_Response
	 */
	private function reordenar_categorias( $categorias ) {
		// Obtener categorías de woocommerce.
		$prepared_args = array(
			'exclude'    => array(),
			'include'    => array(),
			'order'      => 'asc',
			'orderby'    => 'menu_order',
			'product'    => null,
			'hide_empty' => false,
			'number'     => 99,
			'search'     => null,
			'slug'       => null,
			'offset'     => 0,
		);
		$query_result  = get_terms( 'product_cat', $prepared_args );
		$orden         = array();
		foreach ( $categorias as $categoria ) {
			foreach ( $query_result as $key => $term ) {
				$nombre = $term->name;
				if ( strtolower( $nombre ) === strtolower( $categoria ) ) {
					$orden[] = $term;
				}
			}
		}

		$resto = array();
		foreach ( $query_result as $key => $term ) {
			if ( false === array_search( strtolower( $term->name ), array_map( 'strtolower', $categorias ), true ) ) {
				$resto[] = $term;
			}
		}

		$orden = array_merge( $orden, $resto );

		/**
		 * Ordern actual.
		 *
		 * @var WP_Term[] $orden
		 */
//		foreach ( $orden as $key => $item ) {
		for ( $key = count( $orden ); $key > -1; $key-- ) {
			$item    = $orden[ $key ];
			$next_id = null;
			if ( $key + 1 < count( $orden ) ) {
				$next_id = $orden[ $key + 1 ]->term_id;
			}
			wc_set_term_order( $item->term_id, $key, 'product_cat', true );
		}

		return rest_ensure_response( $orden );
	}
}
