<?php
/**
 * Glop Admin  List User
 *
 * @author   Daniel Ruiz
 * @category Admin
 * @package  GLOP/wooglop
 * @since 1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Wooglop_Users_List_Table', false ) ) :

	if ( ! class_exists( 'WP_List_Table' ) ) {
		require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
	}

	/**
	 * Class Woogloop_Users_List_Table
	 */
	class Wooglop_Users_List_Table extends WP_List_Table {
		/**
		 * Lista de usuarios.
		 *
		 * @var array Usuarios.
		 */
		public $usuarios;

		/**
		 * Columnas.
		 *
		 * @return array
		 */
		public static function define_columns() {
			return array(
				'id'           => __( 'ID', 'wooglop' ),
				'glop_usuario' => __( 'Usuario', 'wooglop' ),
				'glop_pass'    => __( 'Contraseña', 'wooglop' ),
				'glop_key'     => __( 'Clave Privada', 'wooglop' ),
				'glop_activar' => __( 'Activar Usuario', 'wooglop' ),
				'glop_options' => null,
			);
		}

		/**
		 * Construct.
		 *
		 * @param array $args Usuarios.
		 *
		 * Wooglop_Users_List_Table constructor.
		 */
		public function __construct( $args = array() ) {
			$this->usuarios = $args['usuarios'];
			parent::__construct(
				array(
					'singular' => __( 'usuario', 'wooglop' ),
					'plural'   => __( 'usuarios', 'wooglop' ),
					'ajax'     => false,
				)
			);
		}

		/**
		 * Preparar items.
		 */
		public function prepare_items() {
			$columns               = $this->get_columns();
			$hidden                = $this->get_hidden_columns();
			$sortable              = $this->get_sortable_columns();
			$this->_column_headers = array( $columns, $hidden, $sortable );
			$this->items           = $this->table_data();
		}

		/**
		 * Override the parent columns method. Defines the columns to use in your listing table
		 *
		 * @return array
		 */
		public function get_columns() {
			return array(
				'id'           => esc_attr__( 'ID', 'wooglop' ),
				'glop_activar' => esc_attr__( '¿Usuario Activo?', 'wooglop' ),
				'glop_usuario' => esc_attr__( 'Usuario', 'wooglop' ),
				'glop_pass'    => esc_attr__( 'Contraseña', 'wooglop' ),
				'glop_key'     => esc_attr__( 'Clave Privada', 'wooglop' ),
				'glop_options' => esc_attr__( 'Opciones', 'wooglop' ),
			);
		}

		/**
		 * Define which columns are hidden
		 *
		 * @return array
		 */
		public function get_hidden_columns() {
			return array();
		}

		/**
		 * Get the table data
		 *
		 * @return array
		 */
		private function table_data() {
			$data = array();
			foreach ( $this->usuarios as $item => $value ) {
				$data[] = array(
					'id'           => $item + 1,
					'glop_usuario' => $value['glop_usuario'],
					'glop_pass'    => $value['glop_pass'],
					'glop_key'     => $value['glop_key'],
					'glop_activar' => $value['glop_activar'],
				);
			}
			return $data;
		}


		/**
		 * Añadir usuario.
		 *
		 * @param string $which Lugar.
		 */
		public function extra_tablenav( $which ) {
			if ( 'bottom' === $which ) {
				$this->add_user_field();
			}

		}

		/**
		 * Botón para añadir usuario.
		 */
		public function add_user_field() {
			?><a href="javascript:show_editar_usuario();" class="button" id="add_user">
				<?php echo esc_attr__( 'Añadir Usuario', 'wooglop' ); ?>
			</a>
			<style>
				.widefat thead td, .widefat thead th,
				.widefat tfoot td, .widefat tfoot th {
					padding-left: 10px;
				}
			</style>
			<?php
		}

		/**
		 * Generate the table navigation above or below the table
		 *
		 * @since 3.1.0
		 * @param string $which Donde.
		 */
		protected function display_tablenav( $which ) {
			?>
			<div class="tablenav <?php echo esc_attr( $which ); ?>">

				<?php
				if ( $this->has_items() || true ) :
					?>
					<div class="alignleft actions bulkactions">
						<?php $this->bulk_actions( $which ); ?>
					</div>
					<?php
				endif;
				$this->extra_tablenav( $which );
				$this->pagination( $which );
				?>

				<br class="clear" />
			</div>
			<?php
		}


		/**
		 * Opciones.
		 *
		 * @param string $item Item.
		 */
		public function column_glop_options( $item ) {
			echo sprintf(
				'<a class="button row-title" href="javascript:;" onclick="fill_and_show_editar_usuario(\'%1$s\', \'%2$s\', \'%3$s\', \'%4$s\', \'%5$s\')">Editar</a> <a class="button row-title" href="javascript:;" onclick="remove_usuario(\'%6$s\')">Eliminar</a>',
				esc_attr( $item['glop_activar'] ),
				esc_attr( $item['id'] ),
				esc_attr( $item['glop_usuario'] ),
				esc_attr( $item['glop_pass'] ),
				esc_attr( $item['glop_key'] ),
				esc_attr( $item['id'] )
			);
		}

		/**
		 * Columnas.
		 *
		 * @param object $item Item actual.
		 * @param string $column_name Columna.
		 *
		 * @return string|true|void
		 */
		public function column_default( $item, $column_name ) {
			switch ( $column_name ) {
				case 'glop_usuario':
				case 'glop_pass':
				case 'glop_key':
				case 'id':
					return $item[ $column_name ];

				case 'glop_activar':
					return '1' === $item['glop_activar'] ? 'Sí' : 'No';
				default:
					return $item;
			}
		}

	}
endif;
