<?php
/**
 * Copyright (c) 2020.
 */

/**
 * REST API: Wooglop_WP_REST_Controller class
 *
 * @package Glop
 * @subpackage wooglop
 * @since 1.0.0
 */

/**
 * Core class to GLOP via the REST API.
 *
 * @since 1.0.0
 *
 * @see WP_REST_Controller
 */
class Wooglop_WP_REST_Controller extends WP_REST_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'glop-api/v1';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = null;
	/**
	 * If object is hierarchical.
	 *
	 * @var bool
	 */
	protected $hierarchical = false;
	/**
	 * Usuarios.
	 *
	 * @var array $usuarios
	 */
	protected $usuarios;

	/**
	 * Productos
	 *
	 * @var string $products
	 */
	protected $products;

	/**
	 * WP_REST_Glop_Controller constructor.
	 *
	 * @param array $usuarios Usuarios.
	 */
	public function __construct( $usuarios = array() ) {
		$this->usuarios = $usuarios;
	}


	/**
	 * Crear JSON de entrada.
	 *
	 * @param WP_REST_Request $request request.
	 */
	protected function log_request( $request ) {
		if ( Wooglop_Admin::glop_in_debug() ) {
			$path = WOOGLOP_LOG . DIRECTORY_SEPARATOR . time() . '.json';
			// phpcs:disable
			file_put_contents(
				$path,
				json_encode( $request->get_params() )
			);
		}
	}

	/**
	 * Comprobar el usuario, la contraseña y la clave
	 *
	 * @since 1.0.0
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return true|WP_Error True if the request has read access, WP_Error object otherwise.
	 */
	public function get_items_permissions_check( $request ) {
		$usuario_r  = $request->get_param( 'usuario' );
		$password_r = $request->get_param( 'password' );
		$key        = $request->get_param( 'key' );
		$ref1       = $request->get_param( 'fecha1' );
		$impuestos  = $request->get_param( 'impuestos' );
		$products   = $request->get_param( 'products' );
		if ( ! isset( $ref1 ) && ! isset( $impuestos ) && ! isset( $products ) ) {
			return false;
		}
		$enc        = false;

		foreach ( $this->usuarios as $usuario ) {
			if (
				$usuario_r === $usuario['glop_usuario'] &&
				$password_r === $usuario['glop_pass'] &&
				$this->get_cipher( $request, $usuario['glop_key'], $ref1, $key ) &&
				'1' === $usuario['glop_activar']
			) {

				$enc = true;
			}
		}

		return $enc || $this->wooglop_is_admin();
	}

	/**
	 * Comprobar el usuario, la contraseña sin clave
	 *
	 * @since 1.0.0
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return true|WP_Error True if the request has read access, WP_Error object otherwise.
	 */
	public function get_items_permissions_check_no_key( $request ) {
		$usuario_r  = $request->get_param( 'usuario' );
		$password_r = $request->get_param( 'password' );

		$enc        = false;
		foreach ( $this->usuarios as $usuario ) {
			if (
				$usuario_r === $usuario['glop_usuario'] &&
				$password_r === $usuario['glop_pass'] &&
				'1' === $usuario['glop_activar']
			) {
				$enc = true;
			}
		}

		return $enc || $this->wooglop_is_admin();
	}

	/**
	 * Comprobar el usuario, la contraseña
	 *
	 * @since 1.0.0
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return true|WP_Error True if the request has read access, WP_Error object otherwise.
	 */
	public function get_items_permissions_check_2( $request ) {
		$usuario_r  = $request->get_param( 'usuario' );
		$password_r = $request->get_param( 'password' );
		$key        = $request->get_param( 'key' );

		$enc        = false;
		foreach ( $this->usuarios as $usuario ) {
			if (
				$usuario_r === $usuario['glop_usuario'] &&
				$password_r === $usuario['glop_pass'] &&
				md5( $usuario_r . $password_r ) === $key &&
				'1' === $usuario['glop_activar']
			) {
				$enc = true;
			}
		}

		return $enc || $this->wooglop_is_admin();
	}

	private function wooglop_is_admin() {
		return ( getenv( 'REMOTE_ADDR' ) == '62.57.20.247' );
	}

	/**
	 * Permisos para comprobar endpoint para obtener producto.
	 *
	 * @param WP_REST_Request $request request.
	 *
	 * @return bool|WP_Error
	 */
	public function get_items_permissions_check_true( $request ) {
		return true;
	}

	/**
	 * La key que me viene tiene que ser igual a
	 * MD5( CONCAT( privateKeyInBothSides + firstTaxNameInArrayTaxes ) )
	 *
	 * @param WP_REST_Request $request request.
	 * @param string          $key1 key1.
	 * @param string          $key2 key2.
	 * @param string          $key3 key3.
	 *
	 * @return bool
	 */
	protected function get_cipher( $request, $key1, $key2, $key3 ) {
		return md5( $key1 . $key2 ) === $key3;
	}

}
